@@ -2,8 +2,7 @@ import re
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core import urlresolvers
-from django.utils.functional import update_wrapper, memoize
-from django.utils.importlib import import_module
+from utkik.utils import import_string, cached_property
 
 
 __all__ = ['handler404', 'handler500', 'include', 'patterns', 'url']
@@ -11,61 +10,23 @@ __all__ = ['handler404', 'handler500', 'include', 'patterns', 'url']
 
 handler404 = 'django.views.defaults.page_not_found'
 handler500 = 'django.views.defaults.server_error'
-view_repr_pat = re.compile(r"\.(\w+)'>$")
-_view_cache = {} # memoize cache
 
 
-def get_view(name):
-    """Import and returns a view from string. You can use the full string path
-    or a shorter notation ``myapp.ViewClass`` if ``myapp`` is in the list of
-    ``INSTALLED_APPS``. We memoize the call to this function as it's unnecessary
-    to resolve and import every time this is called with the same argument.
-    """
-    mod_name, attr = name.rsplit('.', 1)
-    if mod_name in settings.INSTALLED_APPS:
-        try:
-            mod = import_module(mod_name + '.views')
-            return getattr(mod, attr)
-        except Exception:
-            pass
-    mod = import_module(mod_name)
-    return getattr(mod, attr)
-get_view = memoize(get_view, _view_cache, 1)
+class View(object):
+    """A view wrapper that makes function and class based views callable"""
 
-
-class LazyView(object):
-    """Lazy import wrapper for a view function or class. This is what the
-    Django handler will be calling.
-    """
     def __init__(self, view):
-        self._view = view
-        self._view_cache = None
+        self.view = view
+        self.__module__ = view.__module__
+        self.__name__ = view.__name__
+        self.__doc__ = view.__doc_
 
     @property
     def func_name(self):
         """This is just a lame hack to get decent debug info from the lame
         default handler behaviour.
         """
-        name = repr(self.view)
-        m = view_repr_pat.search(name)
-        if m:
-            return m.group(1)
-        return name
-
-    @property
-    def view(self):
-        """Return and cache the view from string or view class/function. Note
-        that in the case of a view class that this is not an instance but the
-        class, instantiation will be done in ``__call__`` method.
-        """
-        if self._view_cache is None:
-            if isinstance(self._view, basestring):
-                self._view_cache = get_view(self._view)
-            else:
-                self._view_cache = self._view
-            # this is our first chance to update the wrapper
-            update_wrapper(self, self._view_cache)
-        return self._view_cache
+        return self.__name__
 
     def __call__(self, request, *args, **kwargs):
         """In case of the wrapped view being determined as a class (that it has
@@ -82,6 +43,27 @@ class LazyView(object):
             'class.' % (self.view.__module__, self.view.__name__)
             )
 
+class LazyView(View):
+    """Lazy import wrapper for a view function or class."""
+
+    def __init__(self, import_name):
+        module, name = import_name.rsplit('.', 1)
+        if module in settings.INSTALLED_APPS:
+            module += '.views'
+        self.__module__ = module
+        self.__name__ = name
+        self.import_name = '%s.%s' % (module, name)
+
+    @cached_property
+    def view(self):
+        """Return and cache the view from string. Note that in the case of a
+        view class that this is not an instance but the class, instantiation
+        will be done in the ``__call__`` method.
+        """
+        view = import_string(self.import_name)
+        self.__doc__ = view.__doc__
+        return view
+
 
 class RegexURLPattern(urlresolvers.RegexURLPattern):
     def __init__(self, regex, callback, default_args=None, name=None):
@@ -90,22 +72,21 @@ class RegexURLPattern(urlresolvers.RegexURLPattern):
         """
         self.regex = re.compile(regex, re.UNICODE)
         self._callback = callback
-        self._callback_cache = None
         self.default_args = default_args or {}
         self.name = name
 
-    @property
+    @cached_property
     def callback(self):
         """This method is a little different from the default
         ``django.core.urlresolvers.RegexURLPattern.callback`` in that we return
-        the callback wrapped in ``LazyView``.
+        the callback wrapped in a ``View`` or ``LazyView``.
         """
-        if self._callback_cache is None:
-            try:
-                self._callback_cache = LazyView(self._callback)
-            except Exception, e:
-                raise urlresolvers.ViewDoesNotExist(e)
-        return self._callback_cache
+        try:
+            if isinstance(self._callback, basestring):
+                return LazyView(self._callback)
+            return View(self._callback)
+        except Exception, e:
+            raise urlresolvers.ViewDoesNotExist(e)
 
 
 def include(arg, namespace=None, app_name=None):
